@@ -12,10 +12,10 @@ import os, re
 from glob import glob
 from kook.utils import read_file, write_file
 
-release = prop('release', '0.8.1')
+release = prop('release', '0.9.0')
 #package = prop('package', 'pyTenjin')
 package = prop('package', 'Tenjin')
-copyright = prop('copyright', "copyright(c) 2007-2009 kuwata-lab.com all rights reserved.")
+copyright = prop('copyright', "copyright(c) 2007-2010 kuwata-lab.com all rights reserved.")
 
 license         = "MIT License"
 #python_basepath = "/Library/Frameworks/Python.framework/Versions/2.4"
@@ -25,10 +25,22 @@ site_packages_path = "%s/site-packages" % python_basepath
 script_file     = "pytenjin"
 library_files   = [ "tenjin.py" ]
 
+kook_default_product = 'test'
 
-@product("package")
+python_bins = [
+    '/opt/local/bin/python2.4',
+    '/usr/local/python/2.5.5/bin/python',
+    '/usr/local/python/2.6.5/bin/python',
+    '/usr/local/python/2.7.0/bin/python',
+    '/usr/local/python/3.0.1/bin/python',
+    '/usr/local/python/3.1/bin/python',
+]
+
+
+@recipe
 @ingreds("examples")
-def task_package(c):
+@spices("-A: create all egg files for each version of python")
+def task_package(c, *args, **kwargs):
     """create package"""
     ## remove files
     pattern = c%"dist/$(package)-$(release)*"
@@ -37,7 +49,7 @@ def task_package(c):
     ## edit files
     repl = (
         (r'\$Release\$', release),
-        (r'\$Release: 0.8.1 $', '$Release: 0.8.1 $' % release),
+        (r'\$Release: 0.9.0 $', '$Release: 0.9.0 $' % release),
         (r'\$Copyright\$', copyright),
         (r'\$Package\$', package),
         (r'\$License\$', license),
@@ -65,12 +77,20 @@ def task_package(c):
         system(c%"tar -cf $(dir).tar $(dir)")
         system(c%"gzip -f9 $(dir).tar")
         ## create *.egg file
-        with chdir(dir):
-            system("python setup.py bdist_egg")
+        def bdist_egg(bin):
+            system("%s setup.py bdist_egg" % bin)
             mv("dist/*.egg", "..")
             rm_rf("build", "dist")
+        with chdir(dir):
+            if kwargs.get('A'):
+                bins = [ x for x in python_bins if re.search(r'2\.[567]', x) ]
+                for bin in bins:
+                    bdist_egg(bin)
+            else:
+                bdist_egg('python')
 
 
+@recipe
 #@ingreds("doc/examples.txt")
 def task_examples(c):
     """create examples"""
@@ -110,6 +130,7 @@ def task_examples(c):
         f.close()
 
 
+@recipe
 def task_uninstall(c):
     #script_file    = "$python_basepath/bin/" + script_file;
     #library_files  = [ os.path.join(site_packages_path, item) for item in library_files ]
@@ -129,11 +150,20 @@ def task_uninstall(c):
         edit(filename, by=repl)
 
 
-def task_test(c):
-    with chdir('test'):
-        system("pykook test")
+@recipe
+@spices('-A: do test with all version of python')
+def task_test(c, *args, **kwargs):
+    if kwargs.get('A'):
+        with chdir('test'):
+            for bin in python_bins:
+                print('******************** ' + bin)
+                system("%s test_all.py" % bin)
+    else:
+        with chdir('test'):
+            system("pykook test")
 
 
+@recipe
 def task_clean(c):
     from glob import glob
     dirs = glob("examples/*");
