@@ -1,10 +1,10 @@
 # -*- coding: utf-8 -*-
 ###
-### $Release: 0.8.1 $
-### copyright(c) 2007-2009 kuwata-lab.com all rights reserved.
+### $Release: 0.9.0 $
+### copyright(c) 2007-2010 kuwata-lab.com all rights reserved.
 ###
 
-import unittest
+from oktest import ok, not_ok, run
 import os, traceback
 import yaml
 
@@ -65,7 +65,7 @@ for item in ['<a&b>', '["c",'+"'d']"]:
       ''', escape(to_str(item)), '''</li>\n''', ));
 #end
 _buf.extend(('''</ul>\n''', ));
-print ''.join(_buf)
+print(''.join(_buf))
 """
 SOURCE2 = """_buf = []; _buf.extend(('''<ul>\\r\\n''', ));\n\
 for item in ['<a&b>', '["c",'+"'d']"]:\n\
@@ -73,7 +73,15 @@ for item in ['<a&b>', '["c",'+"'d']"]:\n\
       ''', escape(to_str(item)), '''</li>\\r\\n''', ));\n\
 #end\n\
 _buf.extend(('''</ul>\\r\\n''', ));\n\
-print ''.join(_buf)
+print(''.join(_buf))
+"""
+SOURCE_N = r"""    1:  _buf = []; _buf.extend(('''<ul>\n''', ));
+    2:  for item in ['<a&b>', '["c",'+"'d']"]:
+    3:      _buf.extend(('''  <li>''', to_str(item), '''
+    4:        ''', escape(to_str(item)), '''</li>\n''', ));
+    5:  #end
+    6:  _buf.extend(('''</ul>\n''', ));
+    7:  print(''.join(_buf))
 """
 
 EXECUTED = r"""<ul>
@@ -111,12 +119,12 @@ items = ['aaa', 'bbb', 'ccc']
 """
 
 
-class MainTest(unittest.TestCase, TestCaseHelper):
+class MainTest(object):
 
-    def setUp(self):
+    def before(self):
         pass
 
-    def tearDown(self):
+    def after(self):
         pass
 
     def _test(self):
@@ -171,10 +179,10 @@ class MainTest(unittest.TestCase, TestCaseHelper):
                         ex = sys.exc_info()[1]
                         lst[0] = ex
                         raise ex
-                self.assertRaises(exception, f1)
+                ok (f1).raises(exception)
                 if errormsg:
                     ex = lst[0]
-                    self.assertTextEqual(errormsg, str(ex))
+                    ok (str(ex)) == errormsg
             else:
                 output = app.execute()
                 #print "*** expected=%s" % repr(expected)
@@ -182,7 +190,7 @@ class MainTest(unittest.TestCase, TestCaseHelper):
                 if python2:
                     if encoding and isinstance(output, unicode):
                         output = output.encode(encoding)
-                self.assertTextEqual(expected, output)
+                ok (output) == expected
         finally:
             try:
                 if filename:
@@ -246,7 +254,7 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self.input    = INPUT[n1:-n2]
         buf = SOURCE.splitlines(True)[1:-2]
         buf.insert(0, "_buf = []\n")
-        buf.append("print ''.join(_buf)\n")
+        buf.append("print(''.join(_buf))\n")
         self.expected = ''.join(buf)
         self._test()
         self.options = "-aconvert"
@@ -256,12 +264,24 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self.options  = "-sb"
         self.input    = INPUT
         n1 = len("_buf = []; ")
-        n2 = len("print ''.join(_buf)\n")
+        n2 = len("print(''.join(_buf))\n")
         self.expected = SOURCE[n1:-n2]
         self._test()
         self.options = "-baconvert"
         self._test()
 
+    def test_number1(self):   # -sN
+        self.options  = "-sN"
+        self.input    = INPUT
+        self.expected = SOURCE_N
+        self._test()
+
+    def test_number2(self):   # -sbN
+        self.options  = "-sbN"
+        self.input    = INPUT
+        self.expected = re.sub(r'\n    7:.*?\n$', "\n", SOURCE_N).replace('_buf = []; ', '')
+        self._test()
+
     def test_cache1(self):   # -a cache
         self.options  = "-a cache"
         self.input    = (
@@ -287,14 +307,14 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         cachename = self.filename + '.cache'
         try:
             self._test()
-            self.assertExists(cachename)
+            ok (cachename).exists()
             import marshal
             dct = marshal.load(open(cachename, 'rb'))
-            self.assertTextEqual(['title', 'items'], dct.get('args'))
+            ok (dct.get('args')) == ['title', 'items']
             if   python2:  expected = "<type 'code'>"
             elif python3:  expected = "<class 'code'>"
-            self.assertEquals(expected, str(type(dct.get('bytecode'))))
-            self.assertTextEqual(script, dct.get('script'))
+            ok (str(type(dct.get('bytecode')))) == expected
+            ok (dct.get('script')) == script
         finally:
             if os.path.exists(cachename):
                 os.unlink(cachename)
@@ -343,7 +363,7 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         '',
         '',
         '#end',
-        'print \'\'.join(_buf)',
+        'print(\'\'.join(_buf))',
         ''))
 
     def test_retrieve(self):  # -S, -a retrieve
@@ -369,7 +389,7 @@ class MainTest(unittest.TestCase, TestCaseHelper):
             '    #end',
             '',
             '#end',
-            'print \'\'.join(_buf)',
+            'print(\'\'.join(_buf))',
             ''))
         self.input = self.input_for_retrieve
         self.expected = expected
@@ -390,7 +410,7 @@ class MainTest(unittest.TestCase, TestCaseHelper):
             '   17:      #end',
             '',
             '   20:  #end',
-            '   21:  print \'\'.join(_buf)',
+            '   21:  print(\'\'.join(_buf))',
             ''))
         self.expected = expected
         self.options = '-SNU'
@@ -408,7 +428,7 @@ class MainTest(unittest.TestCase, TestCaseHelper):
             '            escape(to_str(item)); ',
             '    #end',
             '#end',
-            'print \'\'.join(_buf)',
+            'print(\'\'.join(_buf))',
             ''))
         self.input = self.input_for_retrieve
         self.expected = expected
@@ -426,7 +446,7 @@ class MainTest(unittest.TestCase, TestCaseHelper):
             '   15:              escape(to_str(item)); ',
             '   17:      #end',
             '   20:  #end',
-            '   21:  print \'\'.join(_buf)',
+            '   21:  print(\'\'.join(_buf))',
             ''))
         self.expected = expected
         self.options = '-SNC'
@@ -454,7 +474,7 @@ class MainTest(unittest.TestCase, TestCaseHelper):
             '',
             '',
             '#end',
-            'print \'\'.join(_buf)',
+            'print(\'\'.join(_buf))',
             ''))
         self.input = self.input_for_retrieve
         self.expected = expected
@@ -472,12 +492,12 @@ class MainTest(unittest.TestCase, TestCaseHelper):
         self.expected = EXECUTED
         self.options = '-a render --cache=true'
         self._test()
-        self.assertExists(cachename)
+        ok (cachename).exists()
         # dump test
         try:
             self.filename = False
             self.input    = False
-            self.expected = SOURCE[len('_buf = []; '):-len("print ''.join(_buf)\n")]
+            self.expected = SOURCE[len('_buf = []; '):-len("print(''.join(_buf))\n")]
             #self.options = '-d %s' % cachename
             #self._test()
             self.options = '-a dump %s' % cachename
@@ -641,7 +661,7 @@ class MainTest(unittest.TestCase, TestCaseHelper):
     def test_preamble(self):  # --preamble --postamble
         self.options  = ["-s", "--preamble=_buf=list()", "--postamble=return ''.join(_buf)"]
         self.input    = INPUT
-        self.expected = SOURCE.replace("_buf = []", "_buf=list()").replace("print ", "return ")
+        self.expected = re.sub(r'print\((.*?)\)', r'return \1', SOURCE).replace("_buf = []", "_buf=list()")
         self._test()
 
     def test_xencoding1(self):  # --encoding=encoding
@@ -801,8 +821,6 @@ class MainTest(unittest.TestCase, TestCaseHelper):
             pass
 
 
-remove_unmatched_test_methods(MainTest)
-
-
 if __name__ == '__main__':
-    unittest.main()
+    run(MainTest)
+
