@@ -1,9 +1,9 @@
 ###
-### $Release: 0.8.1 $
-### copyright(c) 2007-2009 kuwata-lab.com all rights reserved.
+### $Release: 0.9.0 $
+### copyright(c) 2007-2010 kuwata-lab.com all rights reserved.
 ###
 
-import unittest
+from oktest import ok, not_ok, run
 import sys, os, re
 
 from testcase_helper import *
@@ -11,16 +11,16 @@ import tenjin
 from tenjin.helpers import *
 
 
-class TemplateTest(unittest.TestCase, TestCaseHelper):
+class TemplateTest(object):
 
     code = TestCaseHelper.generate_testcode(__file__)
     exec(code)
 
 
-    def setUp(self):
+    def before(self):
         pass
 
-    def tearDown(self):
+    def after(self):
         pass
 
     def _test(self):
@@ -72,14 +72,14 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
                 self.fail('%s is expected but not raised.' % exception)
             except Exception:
                 ex = sys.exc_info()[1]
-                self.assertEqual(exception, ex.__class__)
-                #self.assertTrue(isinstance(ex, exception))
+                ok (ex.__class__) == exception
+                #ok (ex).is_a(exception)
                 if errormsg:
                     ## SyntaxError has 'msg' attribute instead of 'message'. Why?
-                    #self.assertEqual(errormsg, ex.message or ex.msg) # failed in python2.3
-                    self.assertEqual(errormsg, ex.args[0])
+                    #ok (ex.message or ex.msg) == errormsg # failed in python2.3
+                    ok (ex.args[0]) == errormsg
                 if filename:
-                    self.assertEqual(filename, ex.filename)
+                    ok (ex.filename) == filename
         else:
             if templateclass:
                 templateclass = eval(templateclass)
@@ -87,10 +87,10 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
             else:
                 template = tenjin.Template(**options)
             script = template.convert(input, filename)
-            self.assertTextEqual(source, script, encoding=encoding)
+            ok (script) == source         # encoding=encoding
             if expected:
                 output = template.render(context)
-                self.assertTextEqual(expected, output, encoding=encoding)
+                ok (output) == expected   # encoding=encoding
 
 
 
@@ -107,7 +107,7 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
 #        context = {'items': items}
 #        output1 = template.render(context)
 #        output2 = template.render(items=items)
-#        self.assertTextEqual(output1, output2)
+#        ok (output2) == output1
 
 
     def test_filename1(self):
@@ -122,8 +122,8 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
             write_file(filename, input)
             template1 = tenjin.Template(filename)
             template2 = tenjin.Template()
-            self.assertTextEqual(template1.script, template2.convert(input))
-            self.assertTextEqual(template1.render(), template2.render())
+            ok (template2.convert(input)) == template1.script
+            ok (template2.render()) == template1.render()
         finally:
             try:
                 os.remove(filename)
@@ -136,15 +136,18 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
         if python2:
             input = "#{base64.encodestring('tenjin')}"
         elif python3:
-            input = "#{base64.encodestring(b'tenjin')}"
+            if hasattr(base64, 'encodebytes'):             # python 3.1 or later
+                input = "#{base64.encodebytes(b'tenjin')}"
+            else:                                          # python 3.0
+                input = "#{base64.encodestring(b'tenjin')}"
         template = tenjin.Template()
         template.convert(input)
         def f1():
             template.render()
-        self.assertRaises(NameError, f1)
+        ok (f1).raises(NameError)
         #tenjin.import_module('base64')
         globals()['base64'] = base64
-        #self.assertNotRaise(f1)
+        #ok (f1).not_raise()
         f1()
 
 
@@ -159,13 +162,13 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
         template.convert(input)
         def f1():
             template.render()
-        self.assertRaises(NameError, f1)
+        ok (f1).raises(NameError)
         if python2:
             #tenjin.import_module(rfc822)
             globals()['rfc822'] = rfc822
         elif python3:
             globals()['email'] = email
-        #self.assertNotRaise(f1)
+        #ok (f1).not_raise()
         f1()
 
 
@@ -174,7 +177,7 @@ class TemplateTest(unittest.TestCase, TestCaseHelper):
             input = "<?py #@ARGS 1x ?>"
             template = tenjin.Template()
             template.convert(input)
-        self.assertRaises(ValueError, f)
+        ok (f).raises(ValueError)
 
     def test_dummy_if_stmt(self):
         input = r"""
@@ -206,11 +209,30 @@ if True: ## dummy
 """[1:]
         t = tenjin.Template()
         actual = t.convert(input)
-        self.assertTextEqual(expected, actual)
+        ok (actual) == expected
 
 
-remove_unmatched_test_methods(TemplateTest)
+    def test_input(self):
+        input = r"""<!DOCTYPE>
+<ul>
+<?py for item in items: ?>
+  <li>#{item}</li>
+<?py #endfor ?>
+</ul>
+"""
+        script = r"""_buf.extend(('''<!DOCTYPE>
+<ul>\n''', ));
+for item in items:
+    _buf.extend(('''  <li>''', to_str(item), '''</li>\n''', ));
+#endfor
+_buf.extend(('''</ul>\n''', ));
+"""
+        t = tenjin.Template("test.foobar.pyhtml", input=input)
+        if "input argument is specified then regard it as template content":
+            ok (t.script) == script
+        if "input argument is specified then timestamp is set to False":
+            ok (t.timestamp) == False
 
 
 if __name__ == '__main__':
-    unittest.main()
+    run(TemplateTest)
