@@ -1,10 +1,10 @@
 ###
-### $Release: 0.8.1 $
-### copyright(c) 2007-2009 kuwata-lab.com all rights reserved.
+### $Release: 0.9.0 $
+### copyright(c) 2007-2010 kuwata-lab.com all rights reserved.
 ###
 
 #import unittest
-import os, sys, difflib, re, traceback
+import os, sys, difflib, re
 import yaml
 
 __all__ = ['TestCaseHelper', 'read_file', 'write_file',
@@ -16,9 +16,13 @@ python2 = sys.version_info[0] == 2
 if python2:
     _unicode = unicode
     _bytes   = str
+    def _is_str(val):
+        return isinstance(val, (str, unicode))
 elif python3:
     _unicode = str
     _bytes   = bytes
+    def _is_str(val):
+        return isinstance(val, (str, bytes))
 
 
 def read_file(filename, mode='rb'):
@@ -52,6 +56,15 @@ def remove_unmatched_test_methods(testcase_class, pattern=None):
 
 class TestCaseHelper:
 
+    @classmethod
+    def remove_tests_except(cls, test_name):
+        if not test_name: return
+        if not test_name.startswith('test_'):
+            test_name = 'test_' + test_name
+        for name in dir(cls):
+            if name.startswith('test_') and name != test_name:
+                delattr(cls, name)
+
     def _testname(self):
         try:
             return self._TestCase__testMethodName
@@ -103,6 +116,49 @@ class TestCaseHelper:
         if os.path.exists(filename):
             self.fail("file exists: " + repr(filename))
 
+    def assertFileExists(self, filename, identifier=None):
+        msg = "File %s expected but nof found." % filename
+        if identifier: msg = "[%s] %s" % (identifier, msg)
+        self.assertTrue(os.path.isfile(filename), msg)
+
+    def assertFileNotExist(self, filename, identifier=None):
+        msg = "File %s expected not to be there but exists." % filename
+        if identifier: msg = "[%s] %s" % (identifier, msg)
+        self.assertFalse(os.path.isfile(filename), msg)
+
+    def assertDirExists(self, dirname, identifier=None):
+        msg = "Directory %s expected but nof found." % dirname
+        if identifier: msg = "[%s] %s" % (identifier, msg)
+        self.assertTrue(os.path.isdir(dirname), msg)
+
+    def assertDirNotExist(self, dirname, identifier=None):
+        msg = "Directory %s expected not to be there but exists." % dirname
+        if identifier: msg = "[%s] %s" % (identifier, msg)
+        self.assertFalse(os.path.isdir(dirname), msg)
+
+    def assertFileNewerThan(self, filename, arg, identifier=None):
+        msg = "File %s expected to be newer thant %s." % (repr(filename), repr(arg))
+        if identifier: msg = "[%s] %s" % (identifier, msg)
+        mtime1 = os.path.getmtime(filename)
+        mtime2 = _is_str(arg) and os.path.getmtime(arg) or arg
+        self.assertTrue(mtime1 > mtime2, msg)
+
+    def assertSameTimestampWith(self, filename, arg, identifier=None):
+        msg = "File %s expected to be the same timestampt with %s." % (repr(filename), repr(arg))
+        if identifier: msg = "[%s] %s" % (identifier, msg)
+        mtime1 = os.path.getmtime(filename)
+        mtime2 = _is_str(arg) and os.path.getmtime(arg) or arg
+        self.assertTrue(mtime1 == mtime2, msg)
+
+    def asertFileOlderThan(self, filename, arg, identifier=None):
+        msg = "File %s expected to be older thant %s." % (repr(filename), repr(arg))
+        if identifier: msg = "[%s] %s" % (identifier, msg)
+        mtime1 = os.path.getmtime(filename)
+        mtime2 = _is_str(arg) and os.path.getmtime(arg) or arg
+        self.assertTrue(mtime1 < mtime2, msg)
+
+    ###
+
     def load_testdata(filename, untabify=True):
         i = filename.rfind('.')
         if filename[i:] != '.yaml' and filename[i:] != '.yml':
@@ -118,27 +174,26 @@ class TestCaseHelper:
         doclist = TestCaseHelper.load_testdata(filename, untabify)
         table = {}
         buf = []
+        a = buf.append
         for doc in doclist:
-            if 'name' not in doc:
-                raise Exception("'name:' is required.")
-            name = doc['name']
-            if name in table:
-                raise Exception("'name: %s' is duplicated." % name)
+            name = doc.get('name')
+            if name is None:  raise Exception("'name:' is required.")
+            if name in table: raise Exception("'name: %s' is duplicated." % name)
             table[name] = doc
-            buf.append(        "def test_%s(self):" % name)
-            for key, val in doc.items():
-                if key[-1] == '*':
-                    key = key[:-1]
-                    val = val.get(lang)
-                if key == 'exception':
-                    buf.append("    self.%s = %s" % (key, val))
-                elif isinstance(val, str):
-                    buf.append('    self.%s = r"""%s"""' % (key, val))
-                else:
-                    buf.append("    self.%s = %s" % (key, repr(val)))
-            buf.append(        "    self.%s()" % testmethod)
-            buf.append(        "#")
-        buf.append('')
+            a(                             'def test_%s(self):' % name)
+            for k, v in doc.items():
+                if k[-1] == '*':
+                    k = k[:-1]
+                    v = v.get(lang)
+                if k == 'exception':     a('    self.%s = %s'        % (k, v))
+                elif isinstance(v, str): a('    self.%s = r"""%s"""' % (k, v))
+                else:                    a('    self.%s = %s'        % (k, repr(v)))
+            a(                             '    self.%s()' % testmethod)
+            a(                             '#')
+        a('')
         code = "\n".join(buf)
         return code
     generate_testcode = staticmethod(generate_testcode)
+
+
+TestCaseHelper.assertRaises2 = TestCaseHelper.assertRaise
